"""Module to contain Phylm class definition"""

from phylm.sources.mtc import Mtc
from phylm.sources.rt import Rt
from phylm.sources.imdb import Imdb

class Phylm:
    """Class to represent a movie"""
    def __init__(self, title, year=None):
        self.title = title
        self.year = year
        self._mtc = self._get_mtc()
        self._rt = self._get_rt()
        self._imdb = self._get_imdb()

    def _get_mtc(self):
        return Mtc(self.title)

    def _get_rt(self):
        return Rt(self.title)

    def _get_imdb(self):
        return Imdb(self.title)

    def genres(self, limit=3):
        """Return the genres"""
        return self._imdb.genres(limit)

    def runtime(self):
        """Return the runtime"""
        return self._imdb.runtime()

    def cast(self, limit=5):
        """Return the cast"""
        return self._imdb.cast(limit)

    def directors(self, limit=3):
        """Return the directors"""
        return self._imdb.directors(limit)

    def plot(self):
        """Return the plot"""
        return self._imdb.plot()

    def imdb_title(self):
        """Return the title from IMDb"""
        return self._imdb.title()

    def imdb_year(self):
        """Return the release year from IMDb"""
        return self._imdb.year()

    def imdb_score(self):
        """Return the IMDb score"""
        return self._imdb.score()

    def imdb_low_confidence(self):
        """Return the IMDb low confidence flag"""
        return self._imdb.low_confidence

    def mtc_title(self):
        """Return the title from Metacritic"""
        return self._mtc.title()

    def mtc_year(self):
        """Return the release year from Metacritic"""
        return self._mtc.year()

    def mtc_score(self):
        """Return the Metacritic score"""
        return self._mtc.score()

    def mtc_low_confidence(self):
        """Return the Metacritic low confidence flag"""
        return self._mtc.low_confidence

    def rt_title(self):
        """Return the title from Rotten Tomatoes"""
        return self._rt.title()

    def rt_year(self):
        """Return the release year from Rotten Tomatoes"""
        return self._rt.year()

    def rt_tomato_score(self):
        """Return the Rotten Tomatoes Tomatometer score"""
        return self._rt.tomato_score()

    def rt_audience_score(self):
        """Return the Rotten Tomatoes Audience score"""
        return self._rt.audience_score()

    def rt_low_confidence(self):
        """Return the Rotten Tomatoes low confidence flag"""
        return self._rt.low_confidence



"""Module to contain some web helper functions"""

from urllib.parse import quote_plus
import requests
from bs4 import BeautifulSoup


def soupify(url):
    """Get a webpage and return the BeautifulSoup representation"""
    user_agent = {'User-agent': 'Mozilla/5.0'}
    search = requests.get(url, headers=user_agent).text
    return BeautifulSoup(search, 'html.parser')


def url_encode(string):
    """URL encode a string to be used in search queries"""
    return quote_plus(string)



"""Tests for the utils module"""

from bs4 import BeautifulSoup
from phylm.utils.web import url_encode, soupify


def test_url_encode():
    """Given a string
    When I call the url_encode function on it
    Then I expect the string to url encoded
    """
    string = 'A film with a / symbol'
    assert url_encode(string) == 'A+film+with+a+%2F+symbol'



def test_soupify_returns_a_bs4_object(requests_mock):
    """Given a url
    When I call the soupify function on it
    Then I expect to receive a Beautiful Soup object
    """
    url = 'https://movies.com/search/the+great+movie'
    resp = '<html><h1>The Great Movie</h1></html>'
    requests_mock.register_uri(
        'GET',
        url,
        request_headers={'User-agent': 'Mozilla/5.0'},
        text=resp,
    )
    soup = soupify(url)
    assert isinstance(soup, BeautifulSoup)
    assert soup.text == 'The Great Movie'



"""Module to contain Rt class definition"""

import json
from phylm.utils.web import soupify, url_encode

class Rt:
    """Class to abstract a Rotten Tomatoes movie search result"""
    def __init__(self, raw_title):
        self.raw_title = raw_title
        self.low_confidence = False
        self._rt_data = self._get_rt_data()

    def _get_rt_data(self):
        url_encoded_film = url_encode(self.raw_title)
        search_url = f"https://www.rottentomatoes.com/search?search={url_encoded_film}"
        soup = soupify(search_url)
        raw = soup.find(id='movies-json')
        if not raw.string:
            return None
        items = json.loads(raw.string)['items']
        if not items:
            return None
        target = None
        for item in items:
            if item['name'].lower() == self.raw_title.lower() and item['tomatometerScore']:
                target = item
                break
        if not target:
            for item in items:
                if item['tomatometerScore'] or item['name'].lower() == self.raw_title.lower():
                    target = item
                    self.low_confidence = True
                    break
        return target

    def title(self):
        """Return the title"""
        if not self._rt_data:
            return None
        return self._rt_data['name']

    def year(self):
        """Return the year"""
        if not self._rt_data:
            return None
        return self._rt_data['releaseYear']

    def tomato_score(self):
        """Return the TomatoScore"""
        if not self._rt_data:
            return None
        return self._rt_data['tomatometerScore'].get('score', 'N/A')

    def audience_score(self):
        """Return the Audience Score"""
        if not self._rt_data:
            return None
        return self._rt_data['audienceScore'].get('score', 'N/A')



import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name='phylm',
    version='3.0.4',
    description='Aggregrate useful data about films',
    long_description=README,
    long_description_content_type="text/markdown",
    author='Dom Batten',
    author_email='dominic.batten@googlemail.com',
    url = 'https://github.com/dbatten5/phylm',
    license="MIT",
    install_requires=[
        'beautifulsoup4',
        'requests',
        'imdbpy',
    ],
    packages=find_packages(),
)



"""Module to contain Mtc class definition"""

import re
from phylm.utils.web import soupify, url_encode

class Mtc:
    """Class to abstract a Metacritic movie search result"""
    def __init__(self, raw_title):
        self.raw_title = raw_title
        self.low_confidence = False
        self._mtc_data = self._get_mtc_data()

    def _get_mtc_data(self):
        url_encoded_film = url_encode(self.raw_title)
        search_url = f"https://www.metacritic.com/search/movie/{url_encoded_film}/results"
        soup = soupify(search_url)
        results = soup.find_all("li", {"class": "result"})
        if not results:
            return None
        target = None
        for result in results:
            result_title = result.find("a").string.strip()
            if result_title.lower() == self.raw_title.lower():
                target = result
                break
        if target is None:
            target = results[0]
            self.low_confidence = True
        return target

    def title(self):
        """Return the title"""
        if not self._mtc_data:
            return None
        return self._mtc_data.find("a").string.strip()

    def year(self):
        """Return the year"""
        if not self._mtc_data:
            return None
        year_meta = self._mtc_data.find("p").string
        return re.search('\d{4}', year_meta).group()

    def score(self):
        """Return the score"""
        if not self._mtc_data:
            return None
        return self._mtc_data.find('span', {'class': 'metascore_w'}).text



"""Module to contain IMDb class definition"""

import imdb

class Imdb:
    """Class to abstract an IMDb movie object"""
    ia = imdb.IMDb()

    def __init__(self, raw_title):
        self.raw_title = raw_title
        self.low_confidence = False
        self._imdb_data = self._get_imdb_data()

    def _get_imdb_data(self):
        results = Imdb.ia.search_movie(self.raw_title)
        if not results:
            return None
        target = None
        for result in results:
            if result['title'].lower() == self.raw_title.lower():
                target = result
                break
        if not target:
            target = results[0]
            self.low_confidence = True
        Imdb.ia.update(target, info=['main'])
        return target

    def title(self):
        """Return the title"""
        if not self._imdb_data:
            return None
        return self._imdb_data['title']

    def genres(self, limit=3):
        """Return the genres"""
        if not self._imdb_data:
            return []
        return self._imdb_data['genres'][:limit]

    def cast(self, limit=5):
        """Return the cast"""
        if not self._imdb_data:
            return []
        return [person['name'] for person in self._imdb_data['cast'][:limit]]

    def runtime(self):
        """Return the runtime"""
        if not self._imdb_data:
            return None
        try:
            return self._imdb_data['runtimes'][0]
        except KeyError:
            return None

    def year(self):
        """Return the year"""
        if not self._imdb_data:
            return None
        return self._imdb_data['year']

    def directors(self, limit=3):
        """Return the directors"""
        if not self._imdb_data:
            return []
        try:
            directors = [person['name'] for person in self._imdb_data['directors'][:limit]]
        except KeyError:
            directors = []
        return directors

    def score(self):
        """Return the score"""
        if not self._imdb_data:
            return None
        try:
            return self._imdb_data['rating']
        except KeyError:
            return None

    def plot(self):
        """Return the plot"""
        if not self._imdb_data:
            return None
        data = self._imdb_data
        if 'plot' not in data.current_info:
            Imdb.ia.update(data, info=['plot'])
        try:
            plot = data['plot'][0].split('::')[0]
        except KeyError:
            plot = ''
        return plot
